@@ -1176,13 +1176,18 @@ static long long parse_uid(char *arg) {
 			return -1;
 		}
 	} else {
-		struct passwd *passwd = getpwnam(arg);
+		struct passwd *passwd;
 
-		if (passwd)
-			uid = passwd->pw_uid;
-		else {
-			SYNTAX_ERROR("Invalid uid or unknown user\n");
-			return -1;
+		for(;;) {
+			errno = 0;
+			passwd = getpwnam(arg);
+			if(passwd) {
+				uid = passwd->pw_uid;
+				break;
+			} else if(errno != EINTR) {
+				SYNTAX_ERROR("Invalid uid or unknown user\n");
+				return -1;
+			}
 		}
 	}
 
@@ -1200,13 +1205,18 @@ static long long parse_gid(char *arg) {
 			return -1;
 		}
 	} else {
-		struct group *group = getgrnam(arg);
+		struct group *group;
 
-		if (group)
-			gid = group->gr_gid;
-		else {
-			SYNTAX_ERROR("Invalid gid or unknown group\n");
-			return -1;
+		for(;;) {
+			errno = 0;
+			group = getgrnam(arg);
+			if (group) {
+				gid = group->gr_gid;
+				break;
+			} else if(errno != EINTR) {
+				SYNTAX_ERROR("Invalid gid or unknown group\n");
+				return -1;
+			}
 		}
 	}
 
@@ -1306,12 +1316,21 @@ static void guid_action(struct action *action, struct dir_ent *dir_ent)
 static int action_parse_mode(struct action_entry *action, int args,
 					char **argv, void **data)
 {
+	char *error;
+	int res;
+
 	if (args == 0) {
 		SYNTAX_ERROR("Mode action expects one or more arguments\n");
 		return 0;
 	}
 
-	return parse_mode_args(source, cur_ptr, args, argv, data);
+	res = parse_mode_args(source, cur_ptr, args, argv, data, &error);
+	if(!res) {
+		fprintf(stderr, "%s", error);
+		free(error);
+	}
+
+	return res;
 }
 
 
@@ -2446,13 +2465,18 @@ static int parse_user_arg(struct test_entry *test, struct atom *atom)
 {
 	struct test_number_arg *number;
 	long long size;
-	struct passwd *uid = getpwnam(atom->argv[0]);
+	struct passwd *uid;
 
-	if(uid)
-		size = uid->pw_uid;
-	else {
-		TEST_SYNTAX_ERROR(test, 1, "Unknown user\n");
-		return 0;
+	for(;;) {
+		errno = 0;
+		uid = getpwnam(atom->argv[0]);
+		if(uid) {
+			size = uid->pw_uid;
+			break;
+		} else if(errno != EINTR) {
+			TEST_SYNTAX_ERROR(test, 1, "Unknown user\n");
+			return 0;
+		}
 	}
 
 	number = MALLOC(sizeof(*number));
@@ -2474,13 +2498,18 @@ static int parse_group_arg(struct test_entry *test, struct atom *atom)
 {
 	struct test_number_arg *number;
 	long long size;
-	struct group *gid = getgrnam(atom->argv[0]);
+	struct group *gid;
 
-	if(gid)
-		size = gid->gr_gid;
-	else {
-		TEST_SYNTAX_ERROR(test, 1, "Unknown group\n");
-		return 0;
+	for(;;) {
+		errno = 0;
+		gid = getgrnam(atom->argv[0]);
+		if(gid) {
+			size = gid->gr_gid;
+			break;
+		} else if(errno != EINTR) {
+			TEST_SYNTAX_ERROR(test, 1, "Unknown group\n");
+			return 0;
+		}
 	}
 
 	number = MALLOC(sizeof(*number));
@@ -3106,7 +3135,7 @@ static int eval_fn(struct atom *atom, struct action_data *action_data)
 static int parse_perm_args(struct test_entry *test, struct atom *atom)
 {
 	int res = 1, mode, op, i;
-	char *arg;
+	char *arg, *error;
 	struct mode_data *head = NULL, *cur = NULL;
 	struct perm_data *perm_data;
 
@@ -3131,15 +3160,18 @@ static int parse_perm_args(struct test_entry *test, struct atom *atom)
 	}
 
 	/* try to parse as an octal number */
-	res = parse_octal_mode_args(source, cur_ptr, atom->args, atom->argv, (void **) &head);
+	res = parse_octal_mode_args(source, cur_ptr, atom->args, atom->argv, (void **) &head, &error);
 	if(res == -1) {
 		/* parse as sym mode argument */
 		for(i = 0; i < atom->args && res; i++, arg = atom->argv[i])
-			res = parse_sym_mode_arg(source, cur_ptr, arg, &head, &cur);
+			res = parse_sym_mode_arg(source, cur_ptr, arg, &head, &cur, &error);
 	}
 
-	if (res == 0)
+	if (res == 0) {
+		fprintf(stderr, "%s", error);
+		free(error);
 		goto finish;
+	}
 
 	/*
 	 * Evaluate the symbolic mode against a permission of 0000 octal
